#if !defined(__PLUGIN_H)
    #define __PLUGIN_H 1
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Progarm name    : plugin.h                                                 */
/*                                                                            */
/* Description     : This header file contains function prototypes and        */
/*                   constants for custom plugin message nodes.               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: Apr 2000                     */
/*                                                                            */
/******************************************************************************/

/* hack to get build to work ok under NT 4.0 */
#if 0  /* no longer seems to be required??? */
	#if !defined(_WIN32_WINNT)
		/* hack to get build to work ok under NT 4.0 */
		#define _WIN32_WINNT 0x0400
	#endif
#endif


#if defined(WIN32)
	#include <windows.h>
	#include <winbase.h>
#endif
#include "unicode.h"		/* Function prototypes for common plugin node funcs   */
#include <BipCos.h> /* contains definition for wchar_t */



#include <malloc.h>
#include <memory.h>
#include <stdio.h>
#include <string.h>
#include "rc.h"

#include <BipCni.h>		/* Function prototypes for Plug-in Node functions     */

// Earlier versions of WMQI (or MQSI as it was then called) had a different structure for the
// vft and exception list.  These structures were simply initialised to binary zeroes.  The 
// following macro is set based on whether or not the version we are building is an early 
// version.  It is used throughout the plugin code to control the initialisation of 
// structures, and is also used to determine the existence of the cniGetEnvironmentMessage
// function.
// This can also be set by the compiler directive, BUILD_VERSION_201
#if defined(CNI_VFT_DEFAULT) && !defined(BUILD_VERSION_201)
  #define POST_201_VERSION
#endif

#include "trace_publ.h"

#if !defined(TRACE_ALLOC)
	#if defined(TRACE_STORAGE)
		#define TRACE_ALLOC(__key, __addr) \
			TRACE_MID(TRACE_INFO, ("STORAGE("__key"): alloc %.8x", __addr));
	#else
		#define TRACE_ALLOC(__key, __addr) 
	#endif
#endif

#if !defined(TRACE_FREE)
	#if defined(TRACE_STORAGE)
		#define TRACE_FREE(__key, __addr) \
			TRACE_MID(TRACE_INFO, ("STORAGE("__key"): free %.8x", __addr));
	#else
		#define TRACE_FREE(__key, __addr) 
	#endif
#endif

#if defined(BUILD_AGGREGATOR_PLUGIN)
    #include "aggregator_constants.h"
#elif defined(BUILD_POSTIT_PLUGIN)
    #include "postit_constants.h"
#elif defined(BUILD_TIMER_PLUGIN)
    #include "timer_constants.h"
#elif defined(BUILD_UUIDGEN_PLUGIN)
    #include "uuidgen_constants.h"
#elif defined(BUILD_GETSQLCODE_PLUGIN)
    #include "getSQLCode_constants.h"
#elif defined(BUILD_TREEWALK_PLUGIN)
    #include "treewalk_constants.h"
#elif defined(BUILD_FILEINPUT_PLUGIN)
    #include "fileInput_constants.h"
#elif defined(BUILD_FLOWMON_PLUGIN)
    #include "flowmon_constants.h"
#endif


#if !defined(IN)
    #define IN
#endif
#if !defined(OUT)
    #define OUT
#endif
#if !defined(INOUT)
    #define INOUT
#endif
#if !defined(TRUE)
    #define TRUE 1
#endif
#if !defined(FALSE)
    #define FALSE 0
#endif
#if !defined(bool_t)
    #define bool_t int
#endif

/* define unicode constants used by plugin.c */

typedef struct pStringConstants {
	CciChar 
			*TRUE_C,
	        *FALSE_C,
	        *NULL_STRING,
	        *ATTR_LABEL;
} pluginStringConstants_t;


/* build static initialisation values for Solaris */
#if defined(_SOLARIS) || defined(_HPUX)

#if defined(DECLARE_PLUGIN_DATA)
CciChar 
		P_TRUE_VAL[sizeof("true")*sizeof(CciChar)] =
{'t','r','u','e',0},
		P_FALSE_VAL[sizeof("false")*sizeof(CciChar)] =
{'f','a','l','s','e',0},
		P_NULL_STRING_VAL[sizeof("")*sizeof(CciChar)] =
{0},
		P_ATTR_LABEL_VAL[sizeof("label")*sizeof(CciChar)] =
{'l','a','b','e','l',0};

#else
extern CciChar 
			P_TRUE_VAL[],
			P_FALSE_VAL[],
			P_NULL_STRING_VAL[],
			P_ATTR_LABEL_VAL[];

#endif
#endif


/* define the initialisation macro */
#if defined(WIN32) || defined(_AIX)
#define PLUGIN_STRING_CONSTANTS_DEFAULTS \
    L"true",                         \
    L"false",                         \
    L"",                         \
	L"label"


#elif defined(_SOLARIS) || defined(_HPUX)

#define PLUGIN_STRING_CONSTANTS_DEFAULTS   \
    P_TRUE_VAL, \
    P_FALSE_VAL, \
    P_NULL_STRING_VAL, \
	P_ATTR_LABEL_VAL
	
#endif


/* finally, define the global initializer structure */

#if defined(DECLARE_PLUGIN_DATA)
    pluginStringConstants_t P = {PLUGIN_STRING_CONSTANTS_DEFAULTS}; 
#else
    extern pluginStringConstants_t P; 
#endif


#define MAX_ATTRIBUTE_NAME_LEN     128
#define MAX_NODE_NAME_LEN          128
#define MAX_TERMINAL_NAME_LEN      256
#define MAX_STRING_SIZE            256
typedef struct attributeTableEntry
{
  void    *valuePtr;                     /* Pointer to value */
  int     type;                         /* Value type */
  CciChar name[MAX_ATTRIBUTE_NAME_LEN]; /* Attribute name */
  bool_t  attributeSet,
	      customAttribute;
  struct attributeTableEntry *nextP;    /* Pointer to next entry */
} attributeTableEntry_t;

typedef struct terminalListEntry
{
  void    *next;                        /* Pointer to next entry */
  void    *handle;                      /* Handle to terminal object */
  CciChar  name[MAX_TERMINAL_NAME_LEN]; /* Terminal name */
} terminalListEntry_t;

typedef struct context
{
  void *nodeObject;                            /* Handle for node object */
  attributeTableEntry_t *attributeListHead;    /* Head of attributes list */
  terminalListEntry_t *inputTerminalListHead;  /* Head of i/p terminal list */
  terminalListEntry_t *outputTerminalListHead; /* Head of o/p terminal list */
  CciChar *nodeName,                           /* Node name */
          *nodeLabel,                          /* Node label */
          *messageFlowName;                    /* Message Flow name */
  char *timerName;                             /* Timer name */
  int nodeType;                                /* Value indicating type of node */
  void *nodeDataP;                             /* Node specific data anchor */
} context_t;

/* return sequence numbers */
#define RSN_001     1
#define RSN_002     2
#define RSN_003     3
#define RSN_004     4
#define RSN_005     5
#define RSN_006     6
#define RSN_007     7
#define RSN_008     8
#define RSN_009     9
#define RSN_010     10
#define RSN_011     11
#define RSN_012     12
#define RSN_013     13
#define RSN_014     14
#define RSN_015     15
#define RSN_016     16
#define RSN_017     17
#define RSN_018     18
#define RSN_019     19
#define RSN_020     20
#define RSN_021     21
#define RSN_022     22
#define RSN_023     23
#define RSN_024     24
#define RSN_025     25
#define RSN_026     26
#define RSN_027     27
#define RSN_028     28
#define RSN_029     29
#define RSN_030     30
#define RSN_031     31
#define RSN_032     32
#define RSN_033     33
#define RSN_034     34
#define RSN_035     35
#define RSN_036     36
#define RSN_037     37
#define RSN_038     38
#define RSN_039     39
#define RSN_040     40

/* Function Prototypes */
int getIntegerAttribute(IN context_t *contextP,
			   	        IN CciChar *attributeName,
						OUT int *valueP);

int getBooleanAttribute(IN context_t *contextP,
			   	        IN CciChar *attributeName,
						OUT bool_t *valueP);

int getStringAttribute(IN context_t *contextP,
			           IN CciChar *attributeName,
					   INOUT int *bufferLengthP,
					   OUT CciChar *bufferP);

int getHexAttribute(IN context_t *contextP,
		            IN CciChar *attributeName,
				    INOUT int *bufferLengthP,
				    OUT CciChar *bufferP);

int convertNameFromHex(IN CciChar *inHexString,
					   INOUT int *bufferLengthP,
					   OUT CciChar *buffer);

attributeTableEntry_t *insertAttributeTableEntry(context_t *context,
                                                 CciChar *nodeName,
												 bool_t customAttribute,
                                                 CciChar *attributeName,
												 int type);

void deleteAttributeTableEntries(context_t *contextP);

void *getInputTerminalHandle(context_t *context,
                             CciChar   *terminalName);
terminalListEntry_t *insertInputTerminalListEntry(context_t *context,
                                                  CciChar *terminalName);
void *getOutputTerminalHandle(context_t *context,
                              CciChar *terminalName);
terminalListEntry_t *insertOutputTerminalListEntry(context_t *context,
                                                   CciChar *terminalName);

void deleteAllTerminalListEntries(context_t *contextP);

void deleteTerminalListEntries(terminalListEntry_t *anchorP);

/* Node Implementation Functions */
#include <BipCci.h>	  /* Data types for Plug-in Node functions     */
void *createNodeContext(CciFactory *factoryObject,
                        CciChar *nodeName,
                        CciNode *nodeObject);

void deleteNodeContext(CciContext *dataP);

int getAttributeName(CciContext *context,
                     int      index,
                     CciChar *buffer,
                     int      bufferSize);

int setAttribute(CciContext *dataP,
				 CciChar *attributeName,
				 CciChar *attributeValue);

int getAttribute(CciContext    *context,
                 CciChar *attributeName,
                 CciChar *buffer,
                 int      bufferSize);

int commonSetAttribute(CciContext *dataP,
				       CciChar *attributeName,
				       CciChar *attributeValue);

void evaluate(CciContext *dataP,
		      CciMessage *destinationList,
		      CciMessage *exceptionList,
		      CciMessage *message);

/* macros */

#define ALLOCATE_STRING_STORAGE(__stringName, __numBytes);   \
        __stringName.buffer = (CciChar *)malloc(__numBytes); \
		memset(__stringName.buffer, 0, __numBytes);          \
        __stringName.bufferLength = __numBytes;

#define FREE_STRING_STORAGE(__stringName);   \
        free(__stringName.buffer);

#endif
